/*
 * Copyright 2011 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.izforge.izpack.panels.process;

import java.io.PrintWriter;
import java.util.Properties;

import com.izforge.izpack.api.data.AutomatedInstallData;
import com.izforge.izpack.api.exception.InstallerException;
import com.izforge.izpack.core.substitutor.VariableSubstitutorImpl;
import com.izforge.izpack.installer.console.PanelConsole;

/**
 * @author Kaloyan Enimanev
 */
public class ProcessPanelConsoleHelper implements PanelConsole
{

    @Override
    public boolean runGeneratePropertiesFile(AutomatedInstallData installData,
            PrintWriter printWriter)
    {
        return true;
    }

    @Override
    public boolean runConsoleFromProperties(AutomatedInstallData installData, Properties p)
    {
        return runConsole(installData);
    }

    @Override
    public boolean runConsole(AutomatedInstallData installData)
    {
        ProcessPanelWorker worker =
                new ProcessPanelWorker(installData, new VariableSubstitutorImpl(
                        installData.getVariables()), installData.getRules());

        ProcessPanelAutomationHelper automationHelper = new ProcessPanelAutomationHelper(worker);
        try
        {
            automationHelper.runAutomated(installData, null);
            return true;
        } catch (InstallerException ex)
        {
            throw new RuntimeException("Process execution has failed : " + ex.getMessage(), ex);
        }
    }

}
